searchData={"items":[{"type":"module","title":"Norm","doc":"Norm is a system for specifying the structure of data. It can be used for\nvalidation and for generation of data. Norm does not provide any set of\npredicates and instead allows you to re-use any of your existing\nvalidations.\n\n```elixir\nimport Norm\n\niex> conform!(123, spec(is_integer() and &(&1 > 0)))\n123\n\niex> conform!(-50, spec(is_integer() and &(&1 > 0)))\n** (Norm.MismatchError) Could not conform input:\nval: -50 fails: &(&1 > 0)\n\niex> user_schema = schema(%{\n...>   user: schema(%{\n...>     name: spec(is_binary()),\n...>     age: spec(is_integer() and &(&1 > 0))\n...>   })\n...> })\niex> input = %{user: %{name: \"chris\", age: 30, email: \"c@keathley.io\"}}\niex> conform!(input, user_schema)\n%{user: %{name: \"chris\", age: 30, email: \"c@keathley.io\"}}\niex> generated_users =\n...>   user_schema\n...>   |> gen()\n...>   |> Enum.take(3)\niex> for g <- generated_users, do: g.user.age > 0 && is_binary(g.user.name)\n[true, true, true]\n```\n\nNorm can also be used to specify contracts for function definitions:\n\n```elixir\ndefmodule Colors do\n  use Norm\n\n  def rgb(), do: spec(is_integer() and &(&1 in 0..255))\n\n  def hex(), do: spec(is_binary() and &String.starts_with?(&1, \"#\"))\n\n  @contract rgb_to_hex(r :: rgb(), g :: rgb(), b :: rgb()) :: hex()\n  def rgb_to_hex(r, g, b) do\n    # ...\n  end\nend\n```","ref":"Norm.html"},{"type":"module","title":"Validation and conforming values - Norm","doc":"Norm validates data by \"conforming\" the value to a specification. If the\nvalues don't conform then a list of errors is returned. There are\n2 functions provided for this `conform/2` and `conform!/2`. If you need to\nreturn a list of well defined errors then you should use `conform/2`.\nOtherwise `conform!/2` is generally more useful. The input data is\nalways passed as the 1st argument to `conform` so that calls to conform\nare easily chainable.","ref":"Norm.html#module-validation-and-conforming-values"},{"type":"module","title":"Predicates and specs - Norm","doc":"Norm does not provide a special set of predicates and instead allows you\nto convert any predicate into a spec with the `spec/1` macro. Predicates\ncan be composed together using the `and` and `or` keywords. You can also\nuse anonymous functions to create specs.\n\n```elixir\nspec(is_binary())\nspec(is_integer() and &(&1 > 0))\nspec(is_binary() and fn str -> String.length(str) > 0 end)\n```\n\nThe data is always passed as the first argument to your predicate so you\ncan use predicates with multiple values like so:\n\n```elixir\niex> defmodule Predicate do\n...>   def greater?(x, y), do: x > y\n...> end\niex> conform!(10, spec(Predicate.greater?(5)))\n10\niex> conform!(3, spec(Predicate.greater?(5)))\n** (Norm.MismatchError) Could not conform input:\nval: 3 fails: Predicate.greater?(5)\n```","ref":"Norm.html#module-predicates-and-specs"},{"type":"module","title":"Tuples and atoms - Norm","doc":"Atoms and tuples can be matched without needing to wrap them in a function.\n\n```elixir\niex> :atom = conform!(:atom, :atom)\n:atom\niex> {1, \"hello\"} = conform!({1, \"hello\"}, {spec(is_integer()), spec(is_binary())})\n{1, \"hello\"}\niex> conform!({1, 2}, {:one, :two})\n** (Norm.MismatchError) Could not conform input:\nval: 1 in: 0 fails: is not an atom.\nval: 2 in: 1 fails: is not an atom.\n```\n\nBecause Norm supports matching on bare tuples we can easily validate functions\nthat return `{:ok, term()}` and `{:error, term()}` tuples. These specifications can be combined with `one_of/1` to create union types.\n\n```elixir\niex> defmodule User do\n...>   defstruct [:name, :age]\n...>\n...>   def get_name(id) do\n...>     case id do\n...>       1 -> {:ok, \"Chris\"}\n...>       2 -> {:ok, \"Alice\"}\n...>       _ -> {:error, \"user does not exist\"}\n...>     end\n...>   end\n...> end\niex> result_spec = one_of([\n...>   {:ok, spec(is_binary())},\n...>   {:error, spec(fn _ -> true end)},\n...> ])\niex> {:ok, _name} = conform!(User.get_name(1), result_spec)\n{:ok, \"Chris\"}\niex> {:ok, \"Alice\"} = conform!(User.get_name(2), result_spec)\n{:ok, \"Alice\"}\niex> {:error, _error} = conform!(User.get_name(-42), result_spec)\n{:error, \"user does not exist\"}\n```","ref":"Norm.html#module-tuples-and-atoms"},{"type":"module","title":"Collections - Norm","doc":"Norm can define collections of values using `coll_of`.\n\n```elixir\niex> conform!([1,2,3], coll_of(spec(is_integer)))\n[1, 2, 3]\n```\n\nCollections can take a number of options:\n\n* `:kind` - predicate function the kind of collection being conformed\n* `:distinct` - boolean value for specifying if the collection should have distinct elements\n* `:min_count` - Minimum element count\n* `:max_count` - Maximum element count\n* `:into` - The output collection the input will be conformed into. If not specified then the input type will be used.\n\n```elixir\niex> conform!([:a, :b, :c], coll_of(spec(is_atom), into: MapSet.new()))\n#MapSet<[:a, :b, :c]>\n```","ref":"Norm.html#module-collections"},{"type":"module","title":"Schemas - Norm","doc":"Norm provides a `schema/1` function for specifying maps and structs:\n\n```elixir\niex> user_schema = schema(%{\n...>   user: schema(%{\n...>     name: spec(is_binary()),\n...>     age: spec(is_integer() and & &1 > 0),\n...>   })\n...> })\niex> conform!(%{user: %{name: \"chris\", age: 31}}, user_schema)\n%{user: %{name: \"chris\", age: 31}}\niex> conform!(%{user: %{name: \"chris\", age: -31}}, user_schema)\n** (Norm.MismatchError) Could not conform input:\nval: -31 in: :user/:age fails: &(&1 > 0)\n```\n\nSchema's are designed to allow systems to grow over time. They provide this\nfunctionality in two ways. The first is that any unspecified fields in the input\nare passed through when conforming the input. The second is that all keys in a\nschema are optional. This means that all of these are valid:\n\n```elixir\niex> user_schema = schema(%{\n...>   name: spec(is_binary()),\n...>   age: spec(is_integer()),\n...> })\niex> conform!(%{}, user_schema)\n%{}\niex> conform!(%{age: 31}, user_schema)\n%{age: 31}\niex> conform!(%{foo: :foo, bar: :bar}, user_schema)\n%{foo: :foo, bar: :bar}\n```\n\nIf you're used to more restrictive systems for managing data these might seem\nlike odd choices. We'll see how to specify required keys when we discuss Selections.\n\n#### Structs\n\nYou can also create specs from structs:\n\n```elixir\ndefmodule User do\n  defstruct [:name, :age]\n\n  def s, do: schema(%__MODULE__{\n      name: spec(is_binary()),\n      age: spec(is_integer())\n    })\nend\n```\n\nThis will ensure that the input is a `User` struct with the key that match\nthe given specification. Its convention to provide a `s()` function in the\nmodule that defines the struct so that schema's can be shared throughout\nyour system.\n\nYou don't need to provide specs for all the keys in your struct. Only the\nspecced keys will be conformed. The remaining keys will be checked for\npresence.\n\n```elixir\ndefmodule Norm.User do\n  defstruct [:name, :age]\nend\niex> user_schema = schema(%Norm.User{})\niex> conform!(%Norm.User{name: \"chris\"}, user_schema)\n```\n\n#### Key semantics\n\nAtom and string keys are matched explicitly and there is no casting that\noccurs when conforming values. If you need to match on string keys you\nshould specify your schema with string keys.\n\nSchemas accomodate growth by disregarding any unspecified keys in the input map.\nThis allows callers to start sending new data over time without coordination\nwith the consuming function.","ref":"Norm.html#module-schemas"},{"type":"module","title":"Selections and optionality - Norm","doc":"We said that all of the fields in a schema are optional. In order to specify\nthe keys that are required in a specific use case we can use a Selection. The\nSelections takes a schema and a list of keys - or keys to lists of keys - that\nmust be present in the schema.\n\n```elixir\niex> user_schema = schema(%{\n...>   user: schema(%{\n...>     name: spec(is_binary()),\n...>     age: spec(is_integer()),\n...>   })\n...> })\niex> just_age = selection(user_schema, [user: [:age]])\niex> conform!(%{user: %{name: \"chris\", age: 31}}, just_age)\n%{user: %{age: 31, name: \"chris\"}}\niex> conform!(%{user: %{name: \"chris\"}}, just_age)\n** (Norm.MismatchError) Could not conform input:\nval: %{name: \"chris\"} in: :user/:age fails: :required\n```\n\nIf you need to mark all fields in a schema as required you can elide the list\nof keys like so:\n\n```elixir\niex> user_schema = schema(%{\n...>   user: schema(%{\n...>     name: spec(is_binary()),\n...>     age: spec(is_integer()),\n...>   })\n...> })\niex> conform!(%{user: %{name: \"chris\", age: 31}}, selection(user_schema))\n%{user: %{name: \"chris\", age: 31}}\n```\n\nSelections are an important tool because they give control over optionality\nback to the call site. This allows callers to determine what they actually need\nand makes schema's much more reusable.","ref":"Norm.html#module-selections-and-optionality"},{"type":"module","title":"Patterns - Norm","doc":"Norm provides a way to specify alternative specs using the `alt/1`\nfunction. This is useful when you need to support multiple schema's or\nmultiple alternative specs.\n\n```elixir\niex> create_event = schema(%{type: spec(&(&1 == :create))})\niex> update_event = schema(%{type: spec(&(&1 == :update))})\niex> event = alt(create: create_event, update: update_event)\niex> conform!(%{type: :create}, event)\n{:create, %{type: :create}}\niex> conform!(%{type: :update}, event)\n{:update, %{type: :update}}\niex> conform!(%{type: :delete}, event)\n** (Norm.MismatchError) Could not conform input:\nval: :delete in: :create/:type fails: &(&1 == :create)\nval: :delete in: :update/:type fails: &(&1 == :update)\n```","ref":"Norm.html#module-patterns"},{"type":"module","title":"Generators - Norm","doc":"Along with validating that data conforms to a given specification, Norm\ncan also use specificiations to generate examples of good data. These\nexamples can then be used for property based testing, local development,\nseeding databases, or any other use case.\n\n```elixir\niex> user_schema = schema(%{\n...>   name: spec(is_binary()),\n...>   age: spec(is_integer() and &(&1 > 0))\n...> })\niex> generated =\n...>   user_schema\n...>   |> gen()\n...>   |> Enum.take(3)\niex> for user <- generated, do: user.age > 0 && is_binary(user.name)\n[true, true, true]\n```\n\nUnder the hood Norm uses StreamData for its data generation. This means\nyou can use your specs in tests like so:\n\n```elixir\ninput_data = schema(%{\"user\" => schema(%{\"name\" => spec(is_binary())})})\n\nproperty \"users can update names\" do\n  check all input <- gen(input_data) do\n    assert :ok == update_user(input)\n  end\nend\n```","ref":"Norm.html#module-generators"},{"type":"module","title":"Built in generators - Norm","doc":"Norm will try to infer the generator to use from the predicate defined in\n`spec`. It looks specifically for the guard clauses used for primitive\ntypes in elixir. Not all of the built in guard clauses are supported yet.\nPRs are very welcome ;).","ref":"Norm.html#module-built-in-generators"},{"type":"module","title":"Guiding generators - Norm","doc":"You may have specs like `spec(fn x -> rem(x, 2) == 0 end)` which check to\nsee that an integer is even or not. This generator expects integer values\nbut there's no way for Norm to determine this. If you try to create\na generator from this spec you'll get an error:\n\n```elixir\ngen(spec(fn x -> rem(x, 2) == 0 end))\n** (Norm.GeneratorError) Unable to create a generator for: fn x -> rem(x, 2) == 0 end\n    (norm) lib/norm.ex:76: Norm.gen/1\n```\n\nYou can guide Norm to the right generator by specifying a guard clause as\nthe first predicate in a spec. If Norm can find the right generator then\nit will use any other predicates as filters in the generator.\n\n```elixir\nEnum.take(gen(spec(is_integer() and fn x -> rem(x, 2) == 0 end)), 5)\n[0, -2, 2, 0, 4]\n```\n\nBut its also possible to create filters that are too specific such as\nthis:\n\n```elixir\ngen(spec(is_binary() and &(&1 =~ ~r/foobarbaz/)))\n```\n\nNorm can determine the generators to use however its incredibly unlikely\nthat Norm will be able to generate data that matches the filter. After 25\nconsecutive unseccessful attempts to generate a good value Norm (StreamData\nunder the hood) will return an error. In these scenarios we can create\na custom generator.","ref":"Norm.html#module-guiding-generators"},{"type":"module","title":"Overriding generators - Norm","doc":"You'll often need to guide your generators into the interesting parts of the\nstate space so that you can easily find bugs. That means you'll want to tweak\nand control your generators. Norm provides an escape hatch for creating your\nown generators with the `with_gen/2` function:\n\n```elixir\nage = spec(is_integer() and &(&1 >= 0))\nreasonable_ages = with_gen(age, StreamData.integer(0..105))\n```\n\nBecause `gen/1` returns a StreamData generator you can compose your generators\nwith other StreamData functions:\n\n```elixir\nage = spec(is_integer() and &(&1 >= 0))\nStreamData.frequency([\n  {3, gen(age)},\n  {1, StreamData.binary()},\n])\n\ngen(age) |> StreamData.map(&Integer.to_string/1) |> Enum.take(5)\n[\"1\", \"1\", \"3\", \"4\", \"1\"]\n```\n\nThis allows you to compose generators however you need to while keeping your\ngeneration co-located with the specification of the data.","ref":"Norm.html#module-overriding-generators"},{"type":"module","title":"Adding contracts to functions - Norm","doc":"You can `conform` data wherever it makes sense to do so in your application.\nBut one of the most common ways to use Norm is to validate a functions arguments\nand return value. Because this is such a common pattern, Norm provides function\nannotations similar to `@spec`:\n\n```elixir\ndefmodule Colors do\n  use Norm\n\n  def rgb(), do: spec(is_integer() and &(&1 in 0..255))\n\n  def hex(), do: spec(is_binary() and &String.starts_with?(&1, \"#\"))\n\n  @contract rgb_to_hex(r :: rgb(), g :: rgb(), b :: rgb()) :: hex()\n  @doc \"Convert an RGB value to its CSS-style hexadecimal notation.\"\n  def rgb_to_hex(r, g, b) do\n    # ...\n  end\nend\n```\n\nIf the arguments for `rgb_to_hex` don't conform to the specification or if\n`rgb_to_hex` does not return a value that conforms to `hex` then an error will\nbe raised.\n\nNote `@contract` must be placed _before_ `@doc` as above for ExDoc and\n`ExUnit.DocTest` to continue working as intended.","ref":"Norm.html#module-adding-contracts-to-functions"},{"type":"function","title":"Norm.alt/1","doc":"Chooses between alternative predicates or patterns. The patterns must be tagged with an atom.\nWhen conforming data to this specification the data is returned as a tuple with the tag.","ref":"Norm.html#alt/1"},{"type":"function","title":"Examples - Norm.alt/1","doc":"iex> conform!(\"foo\", alt(s: spec(is_binary()), a: spec(is_atom())))\n    {:s, \"foo\"}\n    iex> conform!(:foo, alt(s: spec(is_binary()), a: spec(is_atom())))\n    {:a, :foo}\n    iex> conform!(123, alt(num: spec(is_integer()), str: spec(is_binary())))\n    {:num, 123}\n    iex> conform!(\"foo\", alt(num: spec(is_integer()), str: spec(is_binary())))\n    {:str, \"foo\"}\n    iex> conform(true, alt(num: spec(is_integer()), str: spec(is_binary())))\n    {:error, [%{spec: \"is_integer()\", input: true, path: [:num]}, %{spec: \"is_binary()\", input: true, path: [:str]}]}","ref":"Norm.html#alt/1-examples"},{"type":"function","title":"Norm.coll_of/2","doc":"Specifies a generic collection. Collections can be any enumerable type.\n\n`coll_of` takes multiple arguments:\n\n* `:kind` - predicate function the kind of collection being conformed\n* `:distinct` - boolean value for specifying if the collection should have distinct elements\n* `:min_count` - Minimum element count\n* `:max_count` - Maximum element count\n* `:into` - The output collection the input will be conformed into. If not specified then the input type will be used.","ref":"Norm.html#coll_of/2"},{"type":"function","title":"Examples - Norm.coll_of/2","doc":"iex> conform!([:a, :b, :c], coll_of(spec(is_atom())))\n    [:a, :b, :c]\n    iex> conform!([:a, :b, :c], coll_of(spec(is_atom), into: MapSet.new()))\n    MapSet.new([:a, :b, :c])\n    iex> conform!(MapSet.new([:a, :b, :c]), coll_of(spec(is_atom)))\n    MapSet.new([:a, :b, :c])\n    iex> conform!(%{a: 1, b: 2, c: 3}, coll_of({spec(is_atom), spec(is_integer)}))\n    %{a: 1, b: 2, c: 3}\n    iex> conform!([1, 2], coll_of(spec(is_integer), min_count: 1))\n    [1, 2]","ref":"Norm.html#coll_of/2-examples"},{"type":"function","title":"Norm.conform/2","doc":"Verifies that the payload conforms to the specification. A \"success tuple\"\nis returned that contains either the conformed value or the error explanation.","ref":"Norm.html#conform/2"},{"type":"function","title":"Examples: - Norm.conform/2","doc":"iex> conform(42, spec(is_integer()))\n    {:ok, 42}\n    iex> conform(42, spec(fn x -> x == 42 end))\n    {:ok, 42}\n    iex> conform(42, spec(&(&1 >= 0)))\n    {:ok, 42}\n    iex> conform(42, spec(&(&1 >= 100)))\n    {:error, [%{spec: \"&(&1 >= 100)\", input: 42, path: []}]}\n    iex> conform(\"foo\", spec(is_integer()))\n    {:error, [%{spec: \"is_integer()\", input: \"foo\", path: []}]}","ref":"Norm.html#conform/2-examples"},{"type":"function","title":"Norm.conform!/2","doc":"Returns the conformed value or raises a mismatch error.","ref":"Norm.html#conform!/2"},{"type":"function","title":"Examples - Norm.conform!/2","doc":"iex> conform!(42, spec(is_integer()))\n    42\n    iex> conform!(42, spec(is_binary()))\n    ** (Norm.MismatchError) Could not conform input:\n    val: 42 fails: is_binary()","ref":"Norm.html#conform!/2-examples"},{"type":"function","title":"Norm.delegate/1","doc":"Allows encapsulation of a spec in another function. This enables late-binding of\nspecs which enables definition of recursive specs.","ref":"Norm.html#delegate/1"},{"type":"function","title":"Examples: - Norm.delegate/1","doc":"iex> conform!(%{\"value\" => 1, \"left\" => %{\"value\" => 2, \"right\" => %{\"value\" => 4}}}, Norm.Core.DelegateTest.TreeTest.spec())\n    %{\"value\" => 1, \"left\" => %{\"value\" => 2, \"right\" => %{\"value\" => 4}}}\n    iex> conform(%{\"value\" => 1, \"left\" => %{\"value\" => 2, \"right\" => %{\"value\" => 4, \"right\" => %{\"value\" => \"12\"}}}}, Norm.Core.DelegateTest.TreeTest.spec())\n    {:error, [%{input: \"12\", path: [\"left\", \"right\", \"right\", \"value\"], spec: \"is_integer()\"}]}","ref":"Norm.html#delegate/1-examples"},{"type":"function","title":"Norm.gen/1","doc":"Creates a generator from a spec, schema, or selection.","ref":"Norm.html#gen/1"},{"type":"function","title":"Examples - Norm.gen/1","doc":"iex> gen(spec(is_integer())) |> Enum.take(3) |> Enum.all?(&is_integer/1)\n    true\n    iex> gen(spec(is_binary())) |> Enum.take(3) |> Enum.all?(&is_binary/1)\n    true\n    iex> gen(spec(&(&1 > 0)))\n    ** (Norm.GeneratorError) Unable to create a generator for: &(&1 > 0)","ref":"Norm.html#gen/1-examples"},{"type":"function","title":"Norm.map_of/3","doc":"Specifies a map with a type of key and a type of value.","ref":"Norm.html#map_of/3"},{"type":"function","title":"Examples - Norm.map_of/3","doc":"iex> conform!(%{a: 1, b: 2, c: 3}, map_of(spec(is_atom()), spec(is_integer())))\n    %{a: 1, b: 2, c: 3}","ref":"Norm.html#map_of/3-examples"},{"type":"function","title":"Norm.one_of/1","doc":"Chooses between a list of options. Unlike `alt/1` the options don't need to\nbe tagged. Specs are always tested in order and will short circuit if the\ndata passes a validation.","ref":"Norm.html#one_of/1"},{"type":"function","title":"Examples - Norm.one_of/1","doc":"iex> conform!(\"chris\", one_of([spec(is_binary()), :alice]))\n    \"chris\"\n    iex> conform!(:alice, one_of([spec(is_binary()), :alice]))\n    :alice","ref":"Norm.html#one_of/1-examples"},{"type":"function","title":"Norm.schema/1","doc":"Creates a re-usable schema. Schema's are open which means that all keys are\noptional and any non-specified keys are passed through without being conformed.\nIf you need to mark keys as required instead of optional you can use `selection`.","ref":"Norm.html#schema/1"},{"type":"function","title":"Examples - Norm.schema/1","doc":"iex> valid?(%{}, schema(%{name: spec(is_binary())}))\n    true\n    iex> valid?(%{name: \"Chris\"}, schema(%{name: spec(is_binary())}))\n    true\n    iex> valid?(%{name: \"Chris\", age: 31}, schema(%{name: spec(is_binary())}))\n    true\n    iex> valid?(%{age: 31}, schema(%{name: spec(is_binary())}))\n    true\n    iex> valid?(%{name: 123}, schema(%{name: spec(is_binary())}))\n    false\n    iex> conform!(%{}, schema(%{name: spec(is_binary())}))\n    %{}\n    iex> conform!(%{age: 31, name: \"chris\"}, schema(%{name: spec(is_binary())}))\n    %{age: 31, name: \"chris\"}\n    iex> conform!(%{age: 31}, schema(%{name: spec(is_binary())}))\n    %{age: 31}\n    iex> conform!(%{user: %{name: \"chris\"}}, schema(%{user: schema(%{name: spec(is_binary())})}))\n    %{user: %{name: \"chris\"}}","ref":"Norm.html#schema/1-examples"},{"type":"function","title":"Norm.selection/2","doc":"Selections can be used to mark keys on a schema as required. Any unspecified keys\nin the selection are still considered optional. Selections, like schemas,\nare open and allow unspecied keys to be passed through. If no selectors are\nprovided then `selection` defaults to `:all` and recursively marks all keys in\nall nested schema's. If the schema includes internal selections these selections\nwill not be overwritten.","ref":"Norm.html#selection/2"},{"type":"function","title":"Examples - Norm.selection/2","doc":"iex> valid?(%{name: \"chris\"}, selection(schema(%{name: spec(is_binary())}), [:name]))\n    true\n    iex> valid?(%{}, selection(schema(%{name: spec(is_binary())}), [:name]))\n    false\n    iex> valid?(%{user: %{name: \"chris\"}}, selection(schema(%{user: schema(%{name: spec(is_binary())})}), [user: [:name]]))\n    true\n    iex> conform!(%{name: \"chris\"}, selection(schema(%{name: spec(is_binary())}), [:name]))\n    %{name: \"chris\"}\n    iex> conform!(%{name: \"chris\", age: 31}, selection(schema(%{name: spec(is_binary())}), [:name]))\n    %{name: \"chris\", age: 31}","ref":"Norm.html#selection/2-examples"},{"type":"function","title":"Require all keys - Norm.selection/2","doc":"iex> valid?(%{user: %{name: \"chris\"}}, selection(schema(%{user: schema(%{name: spec(is_binary())})})))\n    true","ref":"Norm.html#selection/2-require-all-keys"},{"type":"macro","title":"Norm.spec/1","doc":"Creates a new spec. Specs can be created from any existing predicates or\nanonymous functions. Specs must return a boolean value.\n\nPredicates can be arbitrarily composed using the `and` and `or` keywords.","ref":"Norm.html#spec/1"},{"type":"macro","title":"Examples: - Norm.spec/1","doc":"iex> conform!(21, spec(is_integer()))\n    21\n    iex> conform!(21, spec(is_integer() and &(&1 >= 21)))\n    21\n    iex> conform(\"21\", spec(is_integer() and &(&1 >= 21)))\n    {:error, [%{spec: \"is_integer()\", input: \"21\", path: []}]}\n    iex> conform!(:foo, spec(is_atom() or is_binary()))\n    :foo\n    iex> conform!(\"foo\", spec(is_atom() or is_binary()))\n    \"foo\"\n    iex> conform(21, spec(is_atom() or is_binary()))\n    {:error, [%{spec: \"is_atom()\", input: 21, path: []}, %{spec: \"is_binary()\", input: 21, path: []}]}","ref":"Norm.html#spec/1-examples"},{"type":"function","title":"Norm.valid?/2","doc":"Checks if the value conforms to the spec and returns a boolean.","ref":"Norm.html#valid?/2"},{"type":"function","title":"Examples - Norm.valid?/2","doc":"iex> valid?(42,  spec(is_integer()))\n    true\n    iex> valid?(\"foo\",  spec(is_integer()))\n    false","ref":"Norm.html#valid?/2-examples"},{"type":"function","title":"Norm.with_gen/2","doc":"Overwrites the default generator with a custom generator. The generator\ncan be any valid StreamData generator. This means you can either use Norms\nbuilt in `gen/1` function or you can drop into StreamData directly.","ref":"Norm.html#with_gen/2"},{"type":"function","title":"Examples - Norm.with_gen/2","doc":"iex> Enum.take(gen(with_gen(spec(is_integer()), StreamData.constant(\"hello world\"))), 3)\n    [\"hello world\", \"hello world\", \"hello world\"]","ref":"Norm.html#with_gen/2-examples"},{"type":"module","title":"Norm.Contract","doc":"Design by Contract with Norm.\n\nThis module provides a `@contract` macro that can be used to define specs for arguments and the\nreturn value of a given function.\n\nTo use contracts, call `use Norm` which also imports all `Norm` functions.\n\nSometimes you may want to turn off contracts checking. For example, to skip contracts in production,\nset: `config :norm, enable_contracts: Mix.env != :prod`.","ref":"Norm.Contract.html"},{"type":"module","title":"Examples - Norm.Contract","doc":"defmodule Colors do\n      use Norm\n\n      def rgb(), do: spec(is_integer() and &(&1 in 0..255))\n\n      def hex(), do: spec(is_binary() and &String.starts_with?(&1, \"#\"))\n\n      @contract rgb_to_hex(r :: rgb(), g :: rgb(), b :: rgb()) :: hex()\n      def rgb_to_hex(r, g, b) do\n        # ...\n      end\n    end","ref":"Norm.Contract.html#module-examples"},{"type":"exception","title":"Norm.GeneratorError","doc":"","ref":"Norm.GeneratorError.html"},{"type":"exception","title":"Norm.GeneratorLibraryError","doc":"","ref":"Norm.GeneratorLibraryError.html"},{"type":"exception","title":"Norm.MismatchError","doc":"","ref":"Norm.MismatchError.html"},{"type":"exception","title":"Norm.SpecError","doc":"","ref":"Norm.SpecError.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}